@@ -1,22 +1,26 @@
 #! /usr/bin/env python3
 
+import inspect
+
 import torch
+
 from ..utils import common_functions as c_f
 from ..utils.module_with_records_and_reducer import ModuleWithRecordsReducerAndDistance
 from .mixins import EmbeddingRegularizerMixin
-import inspect
 
 
 class BaseMetricLossFunction(
     EmbeddingRegularizerMixin, ModuleWithRecordsReducerAndDistance
 ):
-    def compute_loss(self, embeddings, labels, indices_tuple=None):
+    def compute_loss(self, embeddings, labels, indices_tuple, ref_emb, ref_labels):
         """
         This has to be implemented and is what actually computes the loss.
         """
         raise NotImplementedError
 
-    def forward(self, embeddings, labels, indices_tuple=None):
+    def forward(
+        self, embeddings, labels=None, indices_tuple=None, ref_emb=None, ref_labels=None
+    ):
         """
         Args:
             embeddings: tensor of size (batch_size, embedding_size)
@@ -27,9 +31,13 @@ class BaseMetricLossFunction(
         Returns: the loss
         """
         self.reset_stats()
-        c_f.assert_embeddings_and_labels_are_same_size(embeddings, labels)
-        labels = labels.to(embeddings.device)
-        loss_dict = self.compute_loss(embeddings, labels, indices_tuple)
+        c_f.check_shapes(embeddings, labels)
+        if labels is not None:
+            labels = c_f.to_device(labels, embeddings)
+        ref_emb, ref_labels = c_f.set_ref_emb(embeddings, labels, ref_emb, ref_labels)
+        loss_dict = self.compute_loss(
+            embeddings, labels, indices_tuple, ref_emb, ref_labels
+        )
         self.add_embedding_regularization_to_loss_dict(loss_dict, embeddings)
         return self.reducer(loss_dict, embeddings, labels)
 
@@ -58,18 +66,25 @@ class BaseMetricLossFunction(
 
 
 class MultipleLosses(torch.nn.Module):
-    def __init__(self, losses, weights=None):
+    def __init__(self, losses, miners=None, weights=None):
         super().__init__()
         self.is_dict = isinstance(losses, dict)
         self.losses = (
             torch.nn.ModuleDict(losses) if self.is_dict else torch.nn.ModuleList(losses)
         )
-        if weights is not None:
-            assert (
-                isinstance(weights, dict)
+
+        if miners is not None:
+            self.assertions_if_not_none(miners, match_all_keys=False)
+            self.miners = (
+                torch.nn.ModuleDict(miners)
                 if self.is_dict
-                else c_f.is_list_or_tuple(weights)
+                else torch.nn.ModuleList(miners)
             )
+        else:
+            self.miners = None
+
+        if weights is not None:
+            self.assertions_if_not_none(weights, match_all_keys=True)
             self.weights = weights
         else:
             self.weights = (
@@ -79,11 +94,35 @@ class MultipleLosses(torch.nn.Module):
             )
 
     def forward(self, embeddings, labels, indices_tuple=None):
+        if self.miners:
+            assert indices_tuple is None
         total_loss = 0
         iterable = self.losses.items() if self.is_dict else enumerate(self.losses)
-        if self.is_dict:
-            for i, loss_func in iterable:
-                total_loss += (
-                    loss_func(embeddings, labels, indices_tuple) * self.weights[i]
-                )
+        for i, loss_func in iterable:
+            curr_indices_tuple = self.get_indices_tuple(
+                i, embeddings, labels, indices_tuple
+            )
+            total_loss += (
+                loss_func(embeddings, labels, curr_indices_tuple) * self.weights[i]
+            )
         return total_loss
+
+    def get_indices_tuple(self, i, embeddings, labels, indices_tuple):
+        if self.miners:
+            if (self.is_dict and i in self.miners) or (
+                not self.is_dict and self.miners[i] is not None
+            ):
+                indices_tuple = self.miners[i](embeddings, labels)
+        return indices_tuple
+
+    def assertions_if_not_none(self, x, match_all_keys):
+        if x is not None:
+            if self.is_dict:
+                assert isinstance(x, dict)
+                if match_all_keys:
+                    assert sorted(list(x.keys())) == sorted(list(self.losses.keys()))
+                else:
+                    assert all(k in self.losses.keys() for k in x.keys())
+            else:
+                assert c_f.is_list_or_tuple(x)
+                assert len(x) == len(self.losses)
